import json
import logging
import aiohttp
from aiohttp import web

from slim.retcode import RETCODE
from ..utils import MetaClassForInit


logger = logging.getLogger(__name__)


class WSHandler(metaclass=MetaClassForInit):
    connections = []
    _on_message = {}

    @classmethod
    def cls_init(cls):
        cls.connections = []
        cls._on_message = {}

    @classmethod
    def route(cls, command):
        def _(obj):
            cls._on_message.setdefault(command, [])
            cls._on_message[command].append(obj)
        return _

    def __init__(self):
        self.ws = None

    async def _handle(self, request):
        ws = web.WebSocketResponse()
        await ws.prepare(request)
        self.connections.append(self)
        self.ws = ws

        async for msg in ws:
            if msg.type == aiohttp.WSMsgType.TEXT:
                if msg.data == 'ws.close':
                    await ws.close()
                else:
                    # request id, head, data
                    rid, command, data = json.loads(msg.data)

                    async def send_json(code, data=NotImplemented):
                        if data is NotImplemented:
                            data = RETCODE.txt_cn.get(code)
                        await ws.send_json([rid, {'code': code, 'data': data}])

                    if command in self._on_message:
                        for i in self._on_message[command]:
                            ret = await i(self, send_json, data)
                            if ret is not None:
                                await send_json(*ret)

            elif msg.type == aiohttp.WSMsgType.ERROR:
                logger.debug('ws connection closed with exception %s' % ws.exception())

        logger.debug('websocket connection closed')
        return ws
